/**
******************************************************************************
* @file  can.c
* @author  IC Applications Department
* @version  V0.8
* @date  2020_06_10
* @brief  This file provides all the CAN firmware functions.
******************************************************************************
* @copy
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, HOLOCENE SHALL NOT BE HELD LIABLE FOR ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
* <h2><center>&copy; COPYRIGHT 2016 HOLOCENE</center></h2>
*/ 
#include "can.h"
#include "HAL_can.h"
u8 flag = 0;
/********************************************************************************************************
**Ϣ void CANM_Init(void)
** CANʼ
** None
** None
**    ע 
********************************************************************************************************/
void CANM_Init(CAN_TypeDef* CANx)
{
    GPIO_InitTypeDef GPIO_InitStructure;
		NVIC_InitTypeDef 	NVIC_InitStructure;
		
		RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);
//		RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);	
	
	switch (*(uint32_t*)&CANx)
    {
        case CAN1_BASE:
						RCC_APB1PeriphClockCmd(RCC_APB1Periph_CAN1, ENABLE);
            GPIO_PinAFConfig(GPIOB, GPIO_Pin_8 | GPIO_Pin_9, GPIO_AF_CAN); //PB8,PB9ΪCANź
				
						GPIO_InitStructure.GPIO_Pin  = GPIO_Pin_9; //CAN RX PB4
						GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
						GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
						GPIO_Init(GPIOB, &GPIO_InitStructure);

						GPIO_InitStructure.GPIO_Pin  = GPIO_Pin_8; //CAN TX PB5
						GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
						GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
						GPIO_Init(GPIOB, &GPIO_InitStructure);
            break;
				case CAN2_BASE:
						RCC_APB1PeriphClockCmd(RCC_APB1Periph_CAN2, ENABLE);
            GPIO_PinAFConfig(GPIOB, GPIO_Pin_4 | GPIO_Pin_5, GPIO_AF_CAN); //PB4,PB5ΪCANź

						GPIO_InitStructure.GPIO_Pin  = GPIO_Pin_4; //CAN RX PB4
						GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
						GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
						GPIO_Init(GPIOB, &GPIO_InitStructure);

						GPIO_InitStructure.GPIO_Pin  = GPIO_Pin_5; //CAN TX PB5
						GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
						GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
						GPIO_Init(GPIOB, &GPIO_InitStructure);
            break;
        default:
            break;
    }
		
		
	
		//NVIC
    NVIC_InitStructure.NVIC_IRQChannel = CAN2_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);

}
/********************************************************************************************************
**Ϣ void CAN_Config(u32 CAN_Pre,CAN_Mode ID,u32 idCode1,u32 idCode2,u32 mask1,u32 mask2)
** CAN
** CAN_Pre:CANͨ
CAN_250K  CAN_500K
fMode:CANͨģʽ
StandardFrame_SingleFilter=0:׼֡˲ģʽ
StandardFrame_SingleFilter=1:չ֡˲ģʽ
ExtendedFrame_SingleFilter=2:׼֡˫˲ģʽ
ExtendedFrame_DoubleFilter=3:չ֡˫˲ģʽ
idCode1մ1
idCode2մ2
mask1մλ1
mask2մλ2
** None
**    ע 
********************************************************************************************************/
void CAN_Config(CAN_TypeDef* CANx, u32 CAN_Pre, CAN_Mode ID, u32 idCode1, u32 idCode2, u32 mask1, u32 mask2)
{
    CAN_Peli_InitTypeDef          CAN_Peli_InitStructure;
    RCC_ClocksTypeDef             RCC_Clocks;
    u32 idCodeTemp1, idMaskTemp1;
    u32 idCodeTemp2, idMaskTemp2;
    CAN_Peli_FilterInitTypeDef    CAN_Peli_FilterInitStructure;

    CAN_ResetMode_Cmd(CANx, ENABLE); // 븴λģʽ
    CAN_Mode_Cmd(CANx, CAN_PELIMode);//CANPeliģʽ

    RCC_GetClocksFreq(&RCC_Clocks);

    CAN_Peli_StructInit(&CAN_Peli_InitStructure);

    CAN_AutoCfg_BaudParam( &CAN_Peli_InitStructure, RCC_Clocks.PCLK1_Frequency, CAN_Pre);

    CAN_Peli_InitStructure.SAM = RESET;//
//		CAN_Peli_InitStructure.SRR = DISABLE;
    CAN_Peli_InitStructure.STM = DISABLE;//DISABLE;//Բģʽ
    CAN_Peli_InitStructure.LOM = DISABLE;//ֻģʽ
    //	CAN_Peli_InitStructure.EWLR = 0x96;
    CAN_Peli_Init(CANx, &CAN_Peli_InitStructure);

    switch(ID)
    {
        case StandardFrame_SingleFilter:
            idCodeTemp1 = idCode1 << (3 + 18);
            idMaskTemp1 = mask1 << (3 + 18);
            CAN_Peli_FilterInitStructure.AFM = CAN_FilterMode_Singal;
            CAN_Peli_FilterInitStructure.CAN_FilterId0 = (idCodeTemp1 >> 24) & 0xff;
            CAN_Peli_FilterInitStructure.CAN_FilterId1 = (idCodeTemp1 >> 16) & 0xff;
            CAN_Peli_FilterInitStructure.CAN_FilterId2 = (idCodeTemp1 >> 8) & 0xff;
            CAN_Peli_FilterInitStructure.CAN_FilterId3 = (idCodeTemp1 >> 0) & 0xff;

            CAN_Peli_FilterInitStructure.CAN_FilterMaskId0 = (idMaskTemp1 >> 24) & 0xff;
            CAN_Peli_FilterInitStructure.CAN_FilterMaskId1 = ((idMaskTemp1 >> 16) & 0xff) | 0x1f;
            CAN_Peli_FilterInitStructure.CAN_FilterMaskId2 = ((idMaskTemp1 >> 8) & 0xff) | 0xff;
            CAN_Peli_FilterInitStructure.CAN_FilterMaskId3 = ((idMaskTemp1 >> 0) & 0xff) | 0xff;
            break;
        case ExtendedFrame_SingleFilter:
            idCodeTemp1 = idCode1 << 3;
            idMaskTemp1 = mask1 << 3;
            CAN_Peli_FilterInitStructure.AFM = CAN_FilterMode_Singal;
            CAN_Peli_FilterInitStructure.CAN_FilterId0 = (idCodeTemp1 >> 24) & 0xff;
            CAN_Peli_FilterInitStructure.CAN_FilterId1 = (idCodeTemp1 >> 16) & 0xff;
            CAN_Peli_FilterInitStructure.CAN_FilterId2 = (idCodeTemp1 >> 8) & 0xff;
            CAN_Peli_FilterInitStructure.CAN_FilterId3 = idCodeTemp1 & 0xff;

            CAN_Peli_FilterInitStructure.CAN_FilterMaskId0 = (idMaskTemp1 >> 24) & 0xff;
            CAN_Peli_FilterInitStructure.CAN_FilterMaskId1 = ((idMaskTemp1 >> 16) & 0xff);
            CAN_Peli_FilterInitStructure.CAN_FilterMaskId2 = ((idMaskTemp1 >> 8) & 0xff);
            CAN_Peli_FilterInitStructure.CAN_FilterMaskId3 = ((idMaskTemp1 >> 0) & 0xff);
            break;
        case StandardFrame_DoubleFilter:
            idCodeTemp1 = idCode1 << (3 + 18);
            idMaskTemp1 = mask1 << (3 + 18);
            idCodeTemp2 = idCode2 << (3 + 18);
            idMaskTemp2 = mask2 << (3 + 18);
            CAN_Peli_FilterInitStructure.AFM = CAN_FilterMode_Double;
            CAN_Peli_FilterInitStructure.CAN_FilterId0 = (idCodeTemp1 >> 24) & 0xff;
            CAN_Peli_FilterInitStructure.CAN_FilterId1 = (idCodeTemp1 >> 16) & 0xff;
            CAN_Peli_FilterInitStructure.CAN_FilterId2 = (idCodeTemp2 >> 24) & 0xff;
            CAN_Peli_FilterInitStructure.CAN_FilterId3 = (idCodeTemp2 >> 16) & 0xff;

            CAN_Peli_FilterInitStructure.CAN_FilterMaskId0 = (idMaskTemp1 >> 24) & 0xff;
            CAN_Peli_FilterInitStructure.CAN_FilterMaskId1 = ((idMaskTemp1 >> 16) & 0xff) | 0x1f;
            CAN_Peli_FilterInitStructure.CAN_FilterMaskId2 = (idMaskTemp2 >> 24) & 0xff;
            CAN_Peli_FilterInitStructure.CAN_FilterMaskId3 = ((idMaskTemp2 >> 16) & 0xff) | 0x1f;
            break;
        case ExtendedFrame_DoubleFilter:
            idCodeTemp1 = idCode1 << (3);
            idMaskTemp1 = mask1 << (3);
            idCodeTemp2 = idCode2 << (3);
            idMaskTemp2 = mask2 << (3);
            CAN_Peli_FilterInitStructure.AFM = CAN_FilterMode_Double;
            CAN_Peli_FilterInitStructure.CAN_FilterId0 = (idCodeTemp1 >> 24) & 0xff;
            CAN_Peli_FilterInitStructure.CAN_FilterId1 = (idCodeTemp1 >> 16) & 0xff;
            CAN_Peli_FilterInitStructure.CAN_FilterId2 = (idCodeTemp2 >> 24) & 0xff;
            CAN_Peli_FilterInitStructure.CAN_FilterId3 = (idCodeTemp2 >> 16) & 0xff;

            CAN_Peli_FilterInitStructure.CAN_FilterMaskId0 = (idMaskTemp1 >> 24) & 0xff;
            CAN_Peli_FilterInitStructure.CAN_FilterMaskId1 = (idMaskTemp1 >> 16) & 0xff;
            CAN_Peli_FilterInitStructure.CAN_FilterMaskId2 = (idMaskTemp2 >> 24) & 0xff;
            CAN_Peli_FilterInitStructure.CAN_FilterMaskId3 = (idMaskTemp2 >> 16) & 0xff;
            break;
        default:
            break;
    }
    CAN_Peli_FilterInit(CANx, &CAN_Peli_FilterInitStructure);
    CAN_Peli_ITConfig(CANx, CAN_IT_RI, ENABLE);
//		CAN_Peli_ITConfig(CANx, CAN_IT_RI|CAN_IT_TI,ENABLE); 
    CAN_ResetMode_Cmd(CANx, DISABLE); //˳λģʽ,빤ģʽ
}
/********************************************************************************************************
**Ϣ void Send_CANFrame(CanTxMsg* TxMessage)
** CANͱ
** CanTxMsg* TxMessage
** None
**    ע 
********************************************************************************************************/
void Send_CANFrame(CAN_TypeDef* CANx, CanTxMsg* TxMessage)
{
    CanPeliTxMsg     CanPeliTxMsgStructure;
    uint32_t ID = 0;
    u32 i;

    if(TxMessage->CANIDtype)  //չ֡
    {
        ID = TxMessage->CANID << 3;

        CanPeliTxMsgStructure.FF   = 0x01;
        CanPeliTxMsgStructure.IDLL = (ID >> 0) & 0xff;
        CanPeliTxMsgStructure.IDLH = (ID >> 8) & 0xff;
        CanPeliTxMsgStructure.IDHL = (ID >> 16) & 0xff;
        CanPeliTxMsgStructure.IDHH = (ID >> 24) & 0xff;
    }
    else											//׼֡
    {
        ID = TxMessage->CANID << 21;

        CanPeliTxMsgStructure.FF   = 0x00;
        CanPeliTxMsgStructure.IDHL = (ID >> 16) & 0xff;
        CanPeliTxMsgStructure.IDHH = (ID >> 24) & 0xff;
    }

    CanPeliTxMsgStructure.DLC  = TxMessage->DLC;
    CanPeliTxMsgStructure.RTR  = TxMessage->RTR;

    for(i = 0; i < 8; i ++ )
    {
        CanPeliTxMsgStructure.Data[i] = *(TxMessage->Data + i);
    }

    CAN_Peli_Transmit(CANx, &CanPeliTxMsgStructure);
}
/********************************************************************************************************
**Ϣ void CAN2_RX1_IRQHandler(void)
** CANжϷ
** None
** None
**    ע 
********************************************************************************************************/
void CAN2_IRQHandler(void)
{

    u32 CAN_IR_STA;

    CAN_IR_STA = CAN2_PELI->IR;
    if(CAN_IR_STA & CAN_IT_RI)
    {
        CAN_Peli_Receive(CAN2, &CanPeliRxMsgStructure);//ճɹ
        flag = 1;
    }
    if(CAN_IR_STA & CAN_IT_BEI)    //
    {
    }
    if(CAN_IR_STA & CAN_IT_DOI  )  //ջ
    {
        CAN2_PELI->CMR  |= 0x08;
        CAN2_PELI->CMR  |= 0x04;
    }
    if(CAN_IR_STA & CAN_IT_ALI  )  // ٲöʧ
    {
    }
}


/**
* @}
*/

/**
* @}
*/

/**
* @}
*/

/*-------------------------(C) COPYRIGHT 2020 HOLOCENE ----------------------*/
